@@ -1,6 +1,7 @@
 /* multifit/multilinear.c
  * 
- * Copyright (C) 2000, 2007 Brian Gough
+ * Copyright (C) 2000, 2007, 2010 Brian Gough
+ * Copyright (C) 2013 Patrick Alken
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,29 +32,16 @@
  *
  */
 
-int
-gsl_multifit_linear (const gsl_matrix * X,
+static int
+multifit_linear_svd (const gsl_matrix * X,
                      const gsl_vector * y,
+                     double tol,
+                     int balance,
+                     size_t * rank,
                      gsl_vector * c,
                      gsl_matrix * cov,
-                     double *chisq, gsl_multifit_linear_workspace * work)
-{
-  size_t rank;
-  int status  = gsl_multifit_linear_svd (X, y, GSL_DBL_EPSILON, &rank, c,
-                                         cov, chisq, work);
-  return status;
-}
-
-/* Handle the general case of the SVD with tolerance and rank */
-
-int
-gsl_multifit_linear_svd (const gsl_matrix * X,
-                         const gsl_vector * y,
-                         double tol,
-                         size_t * rank,
-                         gsl_vector * c,
-                         gsl_matrix * cov,
-                         double *chisq, gsl_multifit_linear_workspace * work)
+                     double *chisq, 
+                     gsl_multifit_linear_workspace * work)
 {
   if (X->size1 != y->size)
     {
@@ -104,9 +92,16 @@ gsl_multifit_linear_svd (const gsl_matrix * X,
 
       gsl_matrix_memcpy (A, X);
 
-      /* Balance the columns of the matrix A */
+      /* Balance the columns of the matrix A if requested */
 
-      gsl_linalg_balance_columns (A, D);
+      if (balance) 
+        {
+          gsl_linalg_balance_columns (A, D);
+        }
+      else
+        {
+          gsl_vector_set_all (D, 1.0);
+        }
 
       /* Decompose A into U S Q^T */
 
@@ -195,29 +190,58 @@ gsl_multifit_linear_svd (const gsl_matrix * X,
 }
 
 int
-gsl_multifit_wlinear (const gsl_matrix * X,
-                      const gsl_vector * w,
-                      const gsl_vector * y,
-                      gsl_vector * c,
-                      gsl_matrix * cov,
-                      double *chisq, gsl_multifit_linear_workspace * work)
+gsl_multifit_linear (const gsl_matrix * X,
+                     const gsl_vector * y,
+                     gsl_vector * c,
+                     gsl_matrix * cov,
+                     double *chisq, gsl_multifit_linear_workspace * work)
 {
   size_t rank;
-  int status  = gsl_multifit_wlinear_svd (X, w, y, GSL_DBL_EPSILON, &rank, c,
-                                          cov, chisq, work);
+  int status  = multifit_linear_svd (X, y, GSL_DBL_EPSILON, 1, &rank, c,
+                                     cov, chisq, work);
   return status;
 }
 
+/* Handle the general case of the SVD with tolerance and rank */
 
 int
-gsl_multifit_wlinear_svd (const gsl_matrix * X,
-                          const gsl_vector * w,
+gsl_multifit_linear_svd (const gsl_matrix * X,
+                         const gsl_vector * y,
+                         double tol,
+                         size_t * rank,
+                         gsl_vector * c,
+                         gsl_matrix * cov,
+                         double *chisq, gsl_multifit_linear_workspace * work)
+{
+  int status = multifit_linear_svd (X, y, tol, 1, rank, c, cov, chisq, work);
+  return status;
+}
+
+int
+gsl_multifit_linear_usvd (const gsl_matrix * X,
                           const gsl_vector * y,
                           double tol,
                           size_t * rank,
                           gsl_vector * c,
                           gsl_matrix * cov,
                           double *chisq, gsl_multifit_linear_workspace * work)
+{
+  int status = multifit_linear_svd (X, y, tol, 0, rank, c, cov, chisq, work);
+  return status;
+}
+
+/* General weighted case */ 
+
+static int
+multifit_wlinear_svd (const gsl_matrix * X,
+                      const gsl_vector * w,
+                      const gsl_vector * y,
+                      double tol,
+                      int balance,
+                      size_t * rank,
+                      gsl_vector * c,
+                      gsl_matrix * cov,
+                      double *chisq, gsl_multifit_linear_workspace * work)
 {
   if (X->size1 != y->size)
     {
@@ -283,9 +307,16 @@ gsl_multifit_wlinear_svd (const gsl_matrix * X,
           }
         }
 
-      /* Balance the columns of the matrix A */
+      /* Balance the columns of the matrix A if requested */
 
-      gsl_linalg_balance_columns (A, D);
+      if (balance) 
+        {
+          gsl_linalg_balance_columns (A, D);
+        }
+      else
+        {
+          gsl_vector_set_all (D, 1.0);
+        }
 
       /* Decompose A into U S Q^T */
 
@@ -340,26 +371,6 @@ gsl_multifit_wlinear_svd (const gsl_matrix * X,
 
       gsl_vector_div (c, D);
 
-      /* Form covariance matrix cov = (Q S^-1) (Q S^-1)^T */
-
-      for (i = 0; i < p; i++)
-        {
-          gsl_vector_view row_i = gsl_matrix_row (QSI, i);
-          double d_i = gsl_vector_get (D, i);
-
-          for (j = i; j < p; j++)
-            {
-              gsl_vector_view row_j = gsl_matrix_row (QSI, j);
-              double d_j = gsl_vector_get (D, j);
-              double s;
-
-              gsl_blas_ddot (&row_i.vector, &row_j.vector, &s);
-
-              gsl_matrix_set (cov, i, j, s / (d_i * d_j));
-              gsl_matrix_set (cov, j, i, s / (d_i * d_j));
-            }
-        }
-
       /* Compute chisq, from residual r = y - X c */
 
       {
@@ -377,6 +388,26 @@ gsl_multifit_wlinear_svd (const gsl_matrix * X,
           }
 
         *chisq = r2;
+
+        /* Form covariance matrix cov = (X^T W X)^-1 = (Q S^-1) (Q S^-1)^T */
+
+        for (i = 0; i < p; i++)
+          {
+            gsl_vector_view row_i = gsl_matrix_row (QSI, i);
+            double d_i = gsl_vector_get (D, i);
+
+            for (j = i; j < p; j++)
+              {
+                gsl_vector_view row_j = gsl_matrix_row (QSI, j);
+                double d_j = gsl_vector_get (D, j);
+                double s;
+
+                gsl_blas_ddot (&row_i.vector, &row_j.vector, &s);
+
+                gsl_matrix_set (cov, i, j, s / (d_i * d_j));
+                gsl_matrix_set (cov, j, i, s / (d_i * d_j));
+              }
+          }
       }
 
       return GSL_SUCCESS;
@@ -384,6 +415,54 @@ gsl_multifit_wlinear_svd (const gsl_matrix * X,
 }
 
 
+int
+gsl_multifit_wlinear (const gsl_matrix * X,
+                      const gsl_vector * w,
+                      const gsl_vector * y,
+                      gsl_vector * c,
+                      gsl_matrix * cov,
+                      double *chisq, gsl_multifit_linear_workspace * work)
+{
+  size_t rank;
+  int status  = multifit_wlinear_svd (X, w, y, GSL_DBL_EPSILON, 1,  &rank, c,
+                                      cov, chisq, work);
+  return status;
+}
+
+int
+gsl_multifit_wlinear_svd (const gsl_matrix * X,
+                          const gsl_vector * w,
+                          const gsl_vector * y,
+                          double tol,
+                          size_t * rank,
+                          gsl_vector * c,
+                          gsl_matrix * cov,
+                          double *chisq, gsl_multifit_linear_workspace * work)
+{
+  int status  = multifit_wlinear_svd (X, w, y, tol, 1, rank, c,
+                                      cov, chisq, work);
+  return status;
+
+}
+
+int
+gsl_multifit_wlinear_usvd (const gsl_matrix * X,
+                           const gsl_vector * w,
+                           const gsl_vector * y,
+                           double tol,
+                           size_t * rank,
+                           gsl_vector * c,
+                           gsl_matrix * cov,
+                           double *chisq, gsl_multifit_linear_workspace * work)
+{
+  int status  = multifit_wlinear_svd (X, w, y, tol, 0, rank, c,
+                                      cov, chisq, work);
+  return status;
+
+}
+
+/* Estimation of values for given x */
+
 int
 gsl_multifit_linear_est (const gsl_vector * x,
                          const gsl_vector * c,
