@@ -27,6 +27,14 @@ double gsl_hypot (const double x, const double y)
   double yabs = fabs(y) ;
   double min, max;
 
+  /* Follow the optional behavior of the ISO C standard and return
+     +Inf when any of the argument is +-Inf, even if the other is NaN.
+     http://pubs.opengroup.org/onlinepubs/009695399/functions/hypot.html */
+  if (isinf(x) || isinf(y)) 
+    {
+      return GSL_POSINF;
+    }
+
   if (xabs < yabs) {
     min = xabs ;
     max = yabs ;
